"""Import upstream source into a branch"""

import os
from shutil import rmtree
from StringIO import StringIO
import tarfile
from unittest import makeSuite

from bzrlib.bzrdir import BzrDir
from bzrlib.osutils import pathjoin
from bzrlib.tests import TestCaseInTempDir

def top_directory(path):
    dirname = os.path.dirname(path)
    last_dirname = dirname
    while True:
        dirname = os.path.dirname(dirname)
        if dirname == '' or dirname == last_dirname:
            return last_dirname
        last_dirname = dirname


def common_directory(names):
    """Determine a single directory prefix from a list of names"""
    possible_prefix = None
    for name in names:
        name_top = top_directory(name)
        if possible_prefix is None:
            possible_prefix = name_top
        else:
            if name_top != possible_prefix:
                return None
    return possible_prefix


def import_tar(tree, tar_input):
    tar_file = tarfile.open('lala', 'r', tar_input)
    prefix = common_directory(tar_file.getnames())
    added = []
    for name in tar_file.getnames():
        relative_path = name
        if prefix is not None:
            relative_path = relative_path[len(prefix)+1:]
        added.append(relative_path)
        tar_file.extract(name, tree.abspath(relative_path))

    for path in added:
        if tree.path2id(path) is None:
            tree.add(path)

class TestImport(TestCaseInTempDir):

    def make_tar(self):
        result = StringIO()
        tar_file = tarfile.open('project-0.1.tar', 'w', result)
        os.mkdir('project-0.1')
        tar_file.add('project-0.1')
        
        f = file('project-0.1/README', 'wb')
        f.write('What?')
        f.close()
        tar_file.add('project-0.1/README')

        f = file('project-0.1/FEEDME', 'wb')
        f.write('Hungry!!')
        f.close()
        tar_file.add('project-0.1/FEEDME')

        tar_file.close()
        rmtree('project-0.1')
        result.seek(0)
        return result

    def make_tar2(self):
        result = StringIO()
        tar_file = tarfile.open('project-0.2.tar', 'w', result)
        os.mkdir('project-0.2')
        tar_file.add('project-0.2')
        
        f = file('project-0.2/README', 'wb')
        f.write('Now?')
        f.close()
        tar_file.add('project-0.2/README')

        tar_file.add('project-0.2/README')

        tar_file.close()
        rmtree('project-0.2')
        result.seek(0)
        return result

    def make_messed_tar(self):
        result = StringIO()
        tar_file = tarfile.open('project-0.1.tar', 'w', result)
        os.mkdir('project-0.1')
        tar_file.add('project-0.1')

        os.mkdir('project-0.2')
        tar_file.add('project-0.2')
        
        f = file('project-0.1/README', 'wb')
        f.write('What?')
        f.close()
        tar_file.add('project-0.1/README')
        tar_file.close()
        rmtree('project-0.1')
        result.seek(0)
        return result

    def test_top_directory(self):
        self.assertEqual(top_directory('ab/b/c'), 'ab')
        self.assertEqual(top_directory('/etc'), '/')

    def test_common_directory(self):
        self.assertEqual(common_directory(['ab/c/d', 'ab/c/e']), 'ab')
        self.assertIs(common_directory(['ab/c/d', 'ac/c/e']), None)

    def test_untar(self):
        tar_file = self.make_tar()
        tree = BzrDir.create_standalone_workingtree('tree')
        import_tar(tree, tar_file)
        self.assertTrue(tree.path2id('README') is not None) 
        self.assertTrue(tree.path2id('FEEDME') is not None)
        
        tar_file = self.make_tar2()
        import_tar(tree, tar_file)
        self.assertTrue(tree.path2id('README') is not None) 
        self.assertTrue(tree.path2id('FEEDME') is None)


    def test_untar2(self):
        tar_file = self.make_messed_tar()
        tree = BzrDir.create_standalone_workingtree('tree')
        import_tar(tree, tar_file)
        self.assertTrue(tree.path2id('project-0.1/README') is not None) 

def test_suite():
    return makeSuite(TestImport)
